"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var preact_1 = require("preact");
var url_1 = require("../client/url");
var provider_1 = require("./provider");
var browser = typeof window !== 'undefined';
var regexps = {};
var location = browser && window.location;
var history = browser && window.history;
/**
 * Router
 */
var Router = /** @class */ (function () {
    function Router(routes) {
        if (routes === void 0) { routes = []; }
        var _this = this;
        this.onPopState = function (_e) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.replace(this.getURL())];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); };
        this.routes = routes;
        this.cache = {};
        if (browser) {
            window.addEventListener('popstate', this.onPopState);
        }
    }
    Router.prototype.push = function (url) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var err;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.possLoad(url)];
                    case 1:
                        err = _a.sent();
                        if (err)
                            return [2 /*return*/, this.error(err)];
                        if (history)
                            window.history.pushState(null, document.title, url);
                        else if (location)
                            location.href = url;
                        return [2 /*return*/];
                }
            });
        });
    };
    Router.prototype.replace = function (url) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var err;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.possLoad(url)];
                    case 1:
                        err = _a.sent();
                        if (err)
                            return [2 /*return*/, this.error(err)];
                        if (history)
                            window.history.replaceState(null, document.title, url);
                        else if (location)
                            location.href = url;
                        return [2 /*return*/];
                }
            });
        });
    };
    // public but shouldn't be used
    Router.prototype._update = function (Component, path) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var url, route;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        url = url_1.parse(document.location.href);
                        route = this.findRoute(url.pathname);
                        // TODO just return here? probably not
                        if (!route) {
                            document.location.pathname = '/404';
                            return [2 /*return*/];
                        }
                        // cache this component
                        this.cache[path] = Component;
                        if (!(route.path === path)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.renderComponent(Component, url, route.params || {})];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    Router.prototype.error = function (err) {
        err.message = '[elmo/router] ' + err.message;
        throw err;
    };
    Router.prototype.possLoad = function (url) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var e_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.load(url)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, null];
                    case 2:
                        e_1 = _a.sent();
                        return [2 /*return*/, e_1];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    Router.prototype.load = function (as) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var url, route, Component, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        url = url_1.parse(as);
                        route = this.findRoute(url.pathname);
                        if (!route)
                            throw new Error("no route found: " + as);
                        _a = this.cache[route.path];
                        if (_a) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.get(route.path)];
                    case 1:
                        _a = (_b.sent());
                        _b.label = 2;
                    case 2:
                        Component = _a;
                        return [4 /*yield*/, this.renderComponent(Component, url, route.params)];
                    case 3:
                        _b.sent();
                        this.cache[route.path] = Component;
                        return [2 /*return*/];
                }
            });
        });
    };
    Router.prototype.get = function (url) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var component, res, code, e_2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        component = null;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        return [4 /*yield*/, window.fetch(url)];
                    case 2:
                        res = _a.sent();
                        return [4 /*yield*/, res.text()];
                    case 3:
                        code = _a.sent();
                        component = this.evaluate(code);
                        return [3 /*break*/, 5];
                    case 4:
                        e_2 = _a.sent();
                        console.error(e_2);
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/, component];
                }
            });
        });
    };
    Router.prototype.renderComponent = function (Component, url, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            function render(props) {
                if (props === void 0) { props = {}; }
                if (!root)
                    return;
                props = Object.assign({}, initial, props);
                firstChild = preact_1.render(preact_1.h(provider_1.default, { router: self, url: url, params: params }, preact_1.h(Component, props)), root, firstChild);
            }
            // TODO: support flash messages
            function redirect(path, _flash) {
                // TODO: error page
                self.push(path).catch(function (err) { return console.error(err); });
                return;
            }
            var root, firstChild, initial, self;
            return tslib_1.__generator(this, function (_a) {
                root = document.getElementById('elmo.page');
                firstChild = root ? root.firstChild : undefined;
                if (!(root instanceof HTMLElement))
                    return [2 /*return*/];
                initial = { render: render, params: params, url: url, redirect: redirect };
                self = this;
                if (Component) {
                    if (!Component.getInitialProps) {
                        return [2 /*return*/, render(initial)];
                    }
                    Component.getInitialProps(initial)
                        .then(function (props) {
                        props && render(props);
                    })
                        .catch(function (err) {
                        // TODO: error page
                        console.error(err);
                    });
                }
                return [2 /*return*/];
            });
        });
    };
    Router.prototype.getURL = function () {
        if (location) {
            return "" + (location.pathname || '') + (location.search || '');
        }
        else {
            return '';
        }
    };
    Router.prototype.evaluate = function (script) {
        var exports = { default: undefined };
        var module = { exports: exports };
        eval(script); // eslint-disable-line no-eval
        if (module.exports && module.exports.default) {
            return module.exports.default;
        }
        else {
            return module.exports;
        }
    };
    Router.prototype.findRoute = function (path) {
        var routes = this.routes;
        for (var i = 0, len = routes.length; i < len; i++) {
            var route = routes[i];
            var regexp = (regexps[route.regexp] =
                regexps[route.regexp] || new RegExp(route.regexp));
            var m = regexp.exec(path);
            if (!m)
                continue;
            var params = {};
            for (var i_1 = 1, len_1 = m.length; i_1 < len_1; ++i_1) {
                var key = route.params[i_1 - 1];
                if (key)
                    params[key] = decodeURIComponent(m[i_1]);
            }
            // trim off any '/' from front or back
            var routePath = route.path.replace(/^\/+|\/+$/g, '');
            routePath = routePath === '' ? routePath : '/' + routePath;
            return {
                path: routePath + '/index.js',
                params: params
            };
        }
    };
    return Router;
}());
exports.default = Router;
