"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rewrite_route_1 = require("../../utils/rewrite-route");
var path_1 = require("path");
function default_1(source, _sourceMap) {
    this.cacheable();
    var _a = this, resourcePath = _a.resourcePath, _b = _a.query, root = _b.root, routes = _b.routes, environment = _b.environment;
    var route = '/' + rewrite_route_1.default(path_1.join(root, 'pages'), '.js', resourcePath);
    // This is a bit confusing because we only want to update once, but
    // need to satisfy 3 update conditions: 1st page load, HMR update,
    // and Router.push(...). We only want to update once when we load
    // the page and when there's an HMR update, but not when we async
    // re-route.
    //
    //   -> only do HMR in development mode
    //   if (module.hot) {
    //   -> accept the hot module, this is a noop when we're booting
    //     module.hot.accept()
    //
    //     -> if we're booting, this will be "idle", "apply" occurs
    //     -> when we've made a change, this will allow us to only
    //     -> target HMR updates
    //     if (module.hot.status() === 'apply') {
    //       const Router = require('elmo/router').default
    //       Router.update(Component, route).catch(console.error)
    //       return
    //     }
    //   }
    //
    //   -> This will not be triggered when we're using Router.push(...)
    //   -> there is no corresponding <script>.
    //   if (document.querySelector('script[src="${route}"]')) {
    //     const Router = require('elmo/router').default
    //     Router.update(Component, route).catch(console.error)
    //   }
    var newSource = "\n  " + source + "\n  ;(function (Component, env, route, routes) {\n    Component = Component.default || Component\n    const Router = require('elmo').Router\n    const router = new Router(routes)\n\n    if (module.hot) {\n      module.hot.accept()\n\n      // this should only occur when we are using\n      // HMR, never when we boot or when we use Router.push(...)\n      if (module.hot.status() === 'apply') {\n        router._update(Component, route).catch(console.error)\n        return\n      }\n    }\n\n    // this should only get rendered on boot, never\n    // during HMR or when we use Router.push(...)\n    if (document.querySelector('script[src=\"'+ route +'\"]')) {\n      router._update(Component, route)\n        .then(function () {\n          // this will signal to prerender.io compatible\n          // clients that the page has rendered successfully\n          window.prerenderReady = true\n        })\n        .catch(console.error)\n    }\n  })(\n    require('" + this.resourcePath + "'),\n    " + JSON.stringify(environment) + ",\n    " + JSON.stringify(route) + ",\n    " + JSON.stringify(routes) + "\n  )\n  ";
    // console.log(newSource)
    this.callback(null, newSource, undefined);
}
exports.default = default_1;
