"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
// import rewrite from '../../utils/rewrite-route'
var preact_1 = require("preact");
var render_to_string = require("preact-render-to-string");
var evaluate_1 = require("../../utils/evaluate");
var path_1 = require("path");
var server_1 = require("../../lib/style/server");
var provider_1 = require("../../lib/provider");
var poss_1 = require("../../utils/poss");
/**
 * Module Exports
 */
var Static = /** @class */ (function () {
    function Static(options) {
        this.options = options;
    }
    Static.prototype.build = function (entries) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var elmoRoot, documentPath, Document, _loop_1, _a, _b, _i, entry, state_1;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        elmoRoot = this.options.elmoRoot;
                        documentPath = Object.keys(entries).find(function (entry) {
                            return path_1.basename(entry, path_1.extname(entry)) === '_document';
                        });
                        if (!documentPath) {
                            return [2 /*return*/, new Error("couldn't find the _document in the entries")];
                        }
                        Document = evaluate_1.default(documentPath, entries[documentPath].source(), {
                            require: require
                        });
                        if (Document instanceof Error) {
                            return [2 /*return*/, Document];
                        }
                        else if (typeof Document !== 'function') {
                            return [2 /*return*/, new Error("document doesn't have a default export")];
                        }
                        // delete the document from the entries
                        delete entries[documentPath];
                        _loop_1 = function (entry) {
                            var Page, common, script, initial, router, props, page, _a, head, style, html;
                            return tslib_1.__generator(this, function (_b) {
                                switch (_b.label) {
                                    case 0:
                                        // ignore non-HTML files
                                        if (path_1.extname(entry) !== '.html')
                                            return [2 /*return*/, "continue"];
                                        Page = evaluate_1.default(entry, entries[entry].source(), { require: require });
                                        if (Page instanceof Error) {
                                            return [2 /*return*/, { value: Page }];
                                        }
                                        else if (typeof Page !== 'function') {
                                            return [2 /*return*/, { value: new Error("\"" + entry + "\" doesn't have a default export") }];
                                        }
                                        common = preact_1.h('script', {
                                            type: 'text/javascript',
                                            src: '/common.js'
                                        });
                                        script = preact_1.h('script', {
                                            type: 'text/javascript',
                                            src: '/' + path_1.join(path_1.dirname(entry), 'index.js')
                                        });
                                        initial = {
                                            redirect: function (_path) { },
                                            render: function (_props) { },
                                            params: {},
                                            url: {
                                                protocol: '',
                                                host: '',
                                                port: 0,
                                                hostname: '',
                                                hash: '',
                                                search: '',
                                                query: {},
                                                pathname: '',
                                                href: ''
                                            }
                                        };
                                        router = {
                                            push: function (_url) { },
                                            replace: function (_url) { }
                                        };
                                        return [4 /*yield*/, getBuildProps(Page, initial)];
                                    case 1:
                                        props = _b.sent();
                                        if (props instanceof Error)
                                            return [2 /*return*/, { value: props }];
                                        page = renderToString(preact_1.h(provider_1.default, { url: initial.url, params: initial.params, router: router }, preact_1.h(Page, props)));
                                        if (page instanceof Error)
                                            return [2 /*return*/, { value: page }];
                                        _a = rewindAll(elmoRoot), head = _a.head, style = _a.style;
                                        html = renderToString(preact_1.h(Document, {
                                            page: page,
                                            common: common,
                                            script: script,
                                            head: head,
                                            style: style,
                                            props: props
                                        }));
                                        if (html instanceof Error)
                                            return [2 /*return*/, { value: html }];
                                        // update the entry
                                        entries[entry] = {
                                            source: function () { return html; },
                                            size: function () { return html.length; }
                                        };
                                        return [2 /*return*/];
                                }
                            });
                        };
                        _a = [];
                        for (_b in entries)
                            _a.push(_b);
                        _i = 0;
                        _c.label = 1;
                    case 1:
                        if (!(_i < _a.length)) return [3 /*break*/, 4];
                        entry = _a[_i];
                        return [5 /*yield**/, _loop_1(entry)];
                    case 2:
                        state_1 = _c.sent();
                        if (typeof state_1 === "object")
                            return [2 /*return*/, state_1.value];
                        _c.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    Static.prototype.apply = function (compiler) {
        var self = this;
        compiler.hooks.thisCompilation.tap('static', function (compilation) {
            compilation.hooks.optimizeAssets.tapAsync('static', function (assets, done) {
                self
                    .build(assets)
                    .then(function (e) {
                    if (e)
                        compilation.errors.push(e);
                    done();
                })
                    .catch(function (e) {
                    console.error('StaticPlugin: uncaught error (watch killed):', e.stack);
                    done(e);
                });
            });
        });
    };
    return Static;
}());
exports.default = Static;
/**
 * Get the initial properties of the static build
 * // TODO: better inputs
 */
// function getBuildProps(Component: any): Promise<Object> {
//   if (Component.getBuildProps) {
//     return Component.getBuildProps({ build: true, path: '', query: {} })
//   } else {
//     return Promise.resolve({})
//   }
// }
/**
 * Render the HTML
 */
// TODO: not exactly sure how to type the Document when there's an evaluate()
// async function renderHTML(
//   Document: any,
//   params: {
//     elmoRoot: string
//     filepath: string
//     Component: any
//     props: any
//   }
// ) {
//   const Component = params.Component
//   const page = Stringify(<Component {...params.props} />)
//   const { head, style } = rewindAll(params.elmoRoot)
//   const docProps = await getBuildProps(Document)
//   // create a virtual DOM tree
//   const vhtml = (
//     <Document
//       page={page}
//       head={head}
//       style={style}
//       props={docProps || {}}
//       filepath={params.filepath}
//     />
//   )
//   const html = '<!DOCTYPE html>' + Stringify(vhtml)
//   // ignore any <style scoped> or <Head /> in the document
//   rewindAll(params.elmoRoot)
//   return html
// }
/**
 * Add the entry file
 */
function getBuildProps(component, initial) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            if (typeof component.getBuildProps === 'function') {
                return [2 /*return*/, poss_1.default(component.getBuildProps(initial))];
            }
            return [2 /*return*/, initial];
        });
    });
}
function renderToString(component) {
    try {
        return render_to_string(component);
    }
    catch (e) {
        // TODO: figure out why uncaught error isn't instanceof Error
        var err = new Error(e.message);
        err.stack = e.stack;
        return err;
    }
}
// Okay so the reason for this is that when we call evaluate
// it may use a non-cached version of <Head /> or <Style/>
// depending on which module we require. These need to be
// pull from the cache to maintain their flush state.
// This function normalizes over all possible places
// you can get these modules from, so no matter where
// we require it we'll get the state.
function rewindAll(elmoRoot) {
    var dist = path_1.join(elmoRoot, 'dist');
    var src = path_1.join(elmoRoot, 'src');
    var cache = require.cache;
    var distHead = path_1.join(dist, 'lib', 'head.js');
    var srcHead = path_1.join(src, 'lib', 'head.ts');
    var distStyle = path_1.join(dist, 'lib', 'style', 'style.js');
    var srcStyle = path_1.join(src, 'lib', 'style', 'style.ts');
    var fn = null;
    var head = [];
    var style = [];
    for (var req in cache) {
        if (!cache[req])
            continue;
        switch (req) {
            case distHead:
                fn = cache[req].exports.default;
                head = head.concat(fn.rewind());
                break;
            case srcHead:
                fn = cache[req].exports.default;
                head = head.concat(fn.rewind());
                break;
            case distStyle:
                fn = cache[req].exports.default;
                style = style.concat(server_1.default(fn.flush()));
                break;
            case srcStyle:
                fn = cache[req].exports.default;
                style = style.concat(server_1.default(fn.flush()));
                break;
        }
    }
    return { head: head, style: style };
}
