"use strict";
/*
 * Module dependencies
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = require("path");
var rewrite_route_1 = require("../utils/rewrite-route");
var readPkg = require("read-pkg-up");
var webpack = require("webpack");
var glob_1 = require("../utils/glob");
/**
 * Environment Variables
 */
function Environment() {
    return Object.keys(process.env).reduce(function (o, k) {
        o['process.env.' + k] = JSON.stringify(process.env[k]);
        return o;
    }, {});
}
// NOTE: these are required client-side
// (e.g. import Router from 'elmo/router')
// these should only be requires that are
// added to the build. the public-facing
// aliases are here to allow us to narrow
// down our elmo typescript loader
function BrowserAliases(_projectRoot, _elmoRoot) {
    return {
        react: 'preact',
        'react-dom': 'preact',
        elmo: path_1.join(__dirname, '..', 'elmo.js'),
        style: path_1.join(__dirname, '..', 'lib', 'style', 'style.js')
    };
}
/**
 * Webpack Config
 */
function Configure(input) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var pkgPath, ELMO_ROOT, target, environment, action, routes, aliases, root, browserAliases, config, bundles, publicBundles, common, rules, plugins, constants;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, readPkg({ cwd: __dirname })];
                case 1:
                    pkgPath = (_a.sent()).path;
                    ELMO_ROOT = path_1.dirname(pkgPath);
                    target = input.target, environment = input.environment, action = input.action, routes = input.routes, aliases = input.aliases;
                    root = path_1.resolve(process.cwd(), input.root || '.');
                    browserAliases = Object.assign(BrowserAliases(root, ELMO_ROOT), aliases);
                    config = {
                        mode: environment,
                        name: 'bundle',
                        context: root,
                        module: { rules: [] },
                        plugins: [],
                        entry: {}
                    };
                    return [4 /*yield*/, glob_1.default(root, path_1.join('bundles', '**'))
                        // filter out any page that contains an underscore in
                        // any part of their path
                    ];
                case 2:
                    bundles = _a.sent();
                    publicBundles = bundles.filter(function (page) {
                        var rel = path_1.relative(path_1.join(root, 'bundles'), page);
                        var parts = rel.split(path_1.sep);
                        for (var i = 0; i < parts.length; i++) {
                            var part = parts[i];
                            if (part[0] === '_')
                                return false;
                        }
                        return true;
                    });
                    // ignore this configuration if we don't have any public bundles
                    if (publicBundles.length === 0) {
                        return [2 /*return*/, null];
                    }
                    common = [];
                    // setup the entries
                    config.entry = publicBundles.reduce(function (obj, page) {
                        var name = rewrite_route_1.default(root, '.js', page);
                        var bundle = [];
                        // add in the common bundles
                        bundle.push.apply(bundle, common);
                        // only include webpack hot middleware when we're
                        // not in production and using the dev server
                        // and we're building for the browser
                        if (action === 'serve' && environment !== 'production') {
                            bundle.push('webpack-hot-middleware/client?path=/__webpack_hmr&noInfo=true&name=' + 'bundle');
                            bundle.push('preact/debug');
                        }
                        obj[name] = bundle.concat(page);
                        return obj;
                    }, {});
                    config.target = 'web';
                    // node
                    config.node = {
                        console: false,
                        global: false,
                        process: false,
                        __filename: false,
                        __dirname: false,
                        Buffer: false,
                        setImmediate: false,
                        dns: false,
                        fs: false,
                        path: true,
                        url: false
                    };
                    // support electron renderer and
                    // web support seamlessly
                    config.externals = [
                        function (_parent, request, fn) {
                            if (request === 'electron') {
                                return fn(null, request);
                            }
                            return fn(null, undefined);
                        }
                    ];
                    // resolves
                    config.resolve = {
                        extensions: ['.ts', '.js', '.tsx', '.jsx'],
                        modules: [path_1.join(root, 'node_modules')],
                        alias: browserAliases
                    };
                    // where to resolve loaders
                    config.resolveLoader = {
                        modules: [path_1.join(root, 'node_modules')]
                    };
                    /**
                     * Output
                     */
                    config.output = {
                        path: '/',
                        publicPath: '/',
                        // [name] ensures the path matches the entry key
                        filename: '[name]',
                        libraryTarget: 'umd'
                    };
                    rules = config.module.rules;
                    // support json files
                    rules.push({
                        test: /\.json$/,
                        use: ['json-loader']
                    });
                    // support loading .jsx files
                    rules.push({
                        test: /\.jsx/,
                        exclude: /(node_modules|bower_components)/,
                        loader: path_1.join(ELMO_ROOT, 'node_modules', 'babel-loader'),
                        options: {
                            plugins: [
                                // TODO: check that these plugins exist
                                path_1.join(ELMO_ROOT, 'node_modules', 'babel-plugin-transform-class-properties'),
                                [
                                    path_1.join(ELMO_ROOT, 'node_modules', 'babel-plugin-transform-react-jsx'),
                                    { pragma: 'h' }
                                ]
                            ]
                        }
                    });
                    // automatically add import { h } from 'preact' to jsx pages
                    // NOTE: we add this to tsx pages, but typescript will
                    // complain about not being able to resolve anyway, but
                    // it makes for simpler tests to include it
                    rules.push({
                        test: /\.[jt]sx/,
                        exclude: /(node_modules|bower_components)/,
                        loader: path_1.join(__dirname, '..', 'server', 'loaders', 'h.js')
                    });
                    // compile our project's typescript files
                    rules.push({
                        test: /\.tsx?$/,
                        exclude: /(node_modules|bower_components)/,
                        loader: path_1.join(__dirname, '..', 'server', 'loaders', 'typescript.js'),
                        query: { environment: environment, target: target, root: root }
                    });
                    // always polyfill to find bugs faster
                    // right now just polyfilling fetch and promise
                    rules.push({
                        test: /\.[tj]sx?$/,
                        include: publicBundles.map(function (page) { return path_1.resolve(root, page); }),
                        loader: path_1.join(__dirname, '..', 'server', 'loaders', 'polyfills.js')
                    });
                    // bundle script for client-side, only added to the top-level pages
                    rules.push({
                        test: /\.[jt]sx?$/,
                        include: publicBundles.map(function (page) { return path_1.resolve(root, page); }),
                        use: {
                            loader: path_1.join(__dirname, '..', 'server', 'loaders', 'bundle.js'),
                            query: { root: root, routes: routes, environment: environment, target: target }
                        }
                    });
                    /**
                     * Reverse plugins, webpack goes from bottom to top
                     * but top to bottom makes more sense in a config
                     */
                    rules = rules.reverse();
                    plugins = config.plugins || [];
                    constants = Environment();
                    // attach the environment to the NODE_ENV
                    constants['process.env.NODE_ENV'] = JSON.stringify(constants['process.env.NODE_ENV'] || environment);
                    // Define environment variables
                    plugins.push(new webpack.DefinePlugin(constants));
                    // Don't write files if there are errors
                    plugins.push(new webpack.NoEmitOnErrorsPlugin());
                    // use names instead of IDs
                    plugins.push(new webpack.NamedModulesPlugin());
                    // setup minification & common chunks
                    config.optimization = {
                        minimize: environment === 'production'
                    };
                    // enable hot module replacement when in dev server in the browser
                    if (action === 'serve' && environment !== 'production') {
                        plugins.push(new webpack.HotModuleReplacementPlugin());
                    }
                    // minify scripts in production in the browser
                    if (environment === 'production') {
                        plugins.push(new webpack.LoaderOptionsPlugin({
                            minimize: true,
                            debug: false
                        }));
                    }
                    config.devtool = false;
                    return [2 /*return*/, config];
            }
        });
    });
}
exports.default = Configure;
